import json
import os
try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.db import models
from django.template.defaultfilters import slugify
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from django.contrib.sites.models import Site

# until the 1.4 shim is released
# https://groups.google.com/d/msg/django-developers/EI-ihJ4CLqw/ZE8kw-JhQtAJ
if not hasattr(settings, "AUTH_USER_MODEL"):
    settings.AUTH_USER_MODEL = "auth.User"

# required external dependencies
from autoslug import AutoSlugField
from model_utils.managers import PassThroughManager

# optional external dependencies
try:
    from licenses.models import License
except:
    License = None

try:
    from imagekit.models import ImageSpecField
    from imagekit.processors import ResizeToFill
except ImportError:
    ResizeToFill = ImageSpecField = None

try:
    from easy_thumbnails.fields import ThumbnailerImageField as ImageField
    custom_image_field = True
except ImportError:
    custom_image_field = False

if not custom_image_field:
    try:
        from sorl.thumbnail import ImageField  # noqa
        custom_image_field = True
    except ImportError:
        custom_image_field = False

if not custom_image_field:
    # image-kit uses the standard ImageField as well
    from django.db.models import ImageField  # noqa

if "taggit" in settings.INSTALLED_APPS:
    from taggit.managers import TaggableManager
else:
    def TaggableManager(blank=True):  # noqa
        return None
try:
    import twitter
except ImportError:
    twitter = None  # noqa

try:
    from embed_video.fields import EmbedVideoField
except ImportError:
    EmbedVideoField = None

# internal imports
from podcasting.managers import EpisodeManager, ShowManager
from podcasting.utils.fields import UUIDField
from podcasting.utils.twitter import can_tweet


def get_show_upload_folder(instance, pathname):
    "A standardized pathname for uploaded files and images."
    root, ext = os.path.splitext(pathname)
    return "img/podcasts/{0}/{1}{2}".format(instance.slug, slugify(root), ext)


def get_episode_upload_folder(instance, pathname):
    "A standardized pathname for uploaded files and images."
    root, ext = os.path.splitext(pathname)
    return "img/podcasts/{0}/episodes/{1}{2}".format(instance.show.slug, slugify(root), ext)


@python_2_unicode_compatible
class Show(models.Model):
    """
    A podcast show, which has many episodes.
    """
    EXPLICIT_CHOICES = (
        (1, _("yes")),
        (2, _("no")),
        (3, _("clean")),
    )
    uuid = UUIDField(_("id"), unique=True)

    created = models.DateTimeField(_("created"), auto_now_add=True, editable=False)
    updated = models.DateTimeField(_("updated"), auto_now=True, editable=False)
    published = models.DateTimeField(_("published"), null=True, blank=True, editable=False)

    site = models.ForeignKey(Site, verbose_name=_('Site'))

    ttl = models.PositiveIntegerField(
        _("ttl"), default=1440,
        help_text=_("""``Time to Live,`` the number of minutes a channel can be
        cached before refreshing."""))

    owner = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name="podcast_shows",
        verbose_name=_("owner"),
        help_text=_("""Make certain the user account has a name and e-mail address."""))

    editor_email = models.EmailField(
        _("editor email"), blank=True,
        help_text=_("Email address of the person responsible for the feed's content."))
    webmaster_email = models.EmailField(
        _("webmaster email"), blank=True,
        help_text=_("Email address of the person responsible for channel publishing."))

    if License:
        license = models.ForeignKey(License, verbose_name=_("license"))
    else:
        license = models.CharField(
            _("license"), max_length=255,
            help_text=_("To publish a podcast to iTunes it is required to set a license type."))

    organization = models.CharField(
        _("organization"), max_length=255,
        help_text=_("Name of the organization, company or Web site producing the podcast."))
    link = models.URLField(_("link"), help_text=_("""URL of either the main website or the
        podcast section of the main website."""))

    enable_comments = models.BooleanField(default=True)

    author_text = models.CharField(
        _("author text"), max_length=255, help_text=_("""
            This tag contains the name of the person or company that is most
            widely attributed to publishing the Podcast and will be
            displayed immediately underneath the title of the Podcast.
            The suggested format is: 'email@example.com (Full Name)'
            but 'Full Name' only, is acceptable. Multiple authors
            should be comma separated."""))

    title = models.CharField(_("title"), max_length=255)
    slug = AutoSlugField(_("slug"), populate_from="title", unique_with="site")

    subtitle = models.CharField(
        _("subtitle"), max_length=255,
        help_text=_("Looks best if only a few words, like a tagline."))

    # If the show is not on iTunes, many fields may be ignored in your user forms
    on_itunes = models.BooleanField(
        _("iTunes"), default=True,
        help_text=_("Checked if the podcast is submitted to iTunes"))

    description = models.TextField(
        _("description"), max_length=4000, help_text=_("""
            This is your chance to tell potential subscribers all about your
            podcast. Describe your subject matter, media format,
            episode schedule, and other relevant info so that they
            know what they'll be getting when they subscribe. In
            addition, make a list of the most relevant search terms
            that you want yourp podcast to match, then build them into
            your description. Note that iTunes removes podcasts that
            include lists of irrelevant words in the itunes:summary,
            description, or itunes:keywords tags. This field can be up
            to 4000 characters."""))

    original_image = ImageField(
        _("image"), upload_to=get_show_upload_folder, blank=True, help_text=_("""
            A podcast must have 1400 x 1400 pixel cover art in JPG or PNG
            format using RGB color space. See our technical spec for
            details. To be eligible for featuring on iTunes Stores,
            choose an attractive, original, and square JPEG (.jpg) or
            PNG (.png) image at a size of 1400x1400 pixels. The image
            will be scaled down to 50x50 pixels at smallest in iTunes.
            For reference see the <a
            href="http://www.apple.com/itunes/podcasts/specs.html#metadata">iTunes
            Podcast specs</a>.<br /><br /> For episode artwork to
            display in iTunes, image must be <a
            href="http://answers.yahoo.com/question/index?qid=20080501164348AAjvBvQ">
            saved to file's <strong>metadata</strong></a> before
            enclosure uploading!"""))

    if ResizeToFill:
        admin_thumb_sm = ImageSpecField(source="original_image",
                                        processors=[ResizeToFill(50, 50)],
                                        options={"quality": 100})
        admin_thumb_lg = ImageSpecField(source="original_image",
                                        processors=[ResizeToFill(450, 450)],
                                        options={"quality": 100})
        img_show_sm = ImageSpecField(source="original_image",
                                     processors=[ResizeToFill(120, 120)],
                                     options={"quality": 100})
        img_show_lg = ImageSpecField(source="original_image",
                                     processors=[ResizeToFill(550, 550)],
                                     options={"quality": 100})
        img_itunes_sm = ImageSpecField(source="original_image",
                                       processors=[ResizeToFill(144, 144)],
                                       options={"quality": 100})
        img_itunes_lg = ImageSpecField(source="original_image",
                                       processors=[ResizeToFill(1400, 1400)],
                                       options={"quality": 100})

    feedburner = models.URLField(
        _("feedburner url"), blank=True,
        help_text=_("""Fill this out after saving this show and at least one
            episode. URL should look like "http://feeds.feedburner.com/TitleOfShow".
            See <a href="http://code.google.com/p/django-podcast/">documentation</a>
            for more. <a href="http://www.feedburner.com/fb/a/ping">Manually ping</a>"""))

    # iTunes specific fields
    explicit = models.PositiveSmallIntegerField(
        _("explicit"), default=1, choices=EXPLICIT_CHOICES,
        help_text=_("``Clean`` will put the clean iTunes graphic by it."))
    redirect = models.URLField(
        _("redirect"), blank=True,
        help_text=_("""The show's new URL feed if changing
            the URL of the current show feed. Must continue old feed for at least
            two weeks and write a 301 redirect for old feed."""))
    keywords = models.CharField(
        _("keywords"), max_length=255, blank=True,
        help_text=_("""A comma-demlimitedlist of up to 12 words for iTunes
            searches. Perhaps include misspellings of the title."""))
    itunes = models.URLField(
        _("itunes store url"), blank=True,
        help_text=_("""Fill this out after saving this show and at least one
            episode. URL should look like:
            "http://phobos.apple.com/WebObjects/MZStore.woa/wa/viewPodcast?id=000000000".
            See <a href="http://code.google.com/p/django-podcast/">documentation</a> for more."""))

    twitter_tweet_prefix = models.CharField(
        _("Twitter tweet prefix"), max_length=80,
        help_text=_("Enter a short ``tweet_text`` prefix for new episodes on this show."),
        blank=True)

    objects = PassThroughManager.for_queryset_class(ShowManager)()
    tags = TaggableManager(blank=True)

    class Meta:
        verbose_name = _("Show")
        verbose_name_plural = _("Shows")
        ordering = ("organization", "slug")

    def __str__(self):
        return self.title

    def get_share_url(self):
        return "http://{0}{1}".format(Site.objects.get_current(), self.get_absolute_url())

    def get_absolute_url(self):
        return reverse("podcasting_show_detail", kwargs={"slug": self.slug})

    @property
    def current_episode(self):
        try:
            return self.episode_set.published().order_by("-published")[0]
        except IndexError:
            return None


@python_2_unicode_compatible
class Episode(models.Model):
    """
    An individual podcast episode and it's unique attributes.
    """
    SIXTY_CHOICES = tuple((x, x) for x in range(60))
    uuid = UUIDField("ID", unique=True)

    created = models.DateTimeField(_("created"), auto_now_add=True, editable=False)
    updated = models.DateTimeField(_("updated"), auto_now=True, editable=False)
    published = models.DateTimeField(_("published"), null=True, blank=True, editable=False)

    show = models.ForeignKey(Show, verbose_name=_("Podcast"))

    enable_comments = models.BooleanField(default=True)

    author_text = models.CharField(_("author text"), max_length=255, blank=True, help_text=_("""
        The person or musician name(s) featured on this specific episode.
        The suggested format is: 'email@example.com (Full Name)' but 'Full Name' only,
        is acceptable. Multiple authors should be comma separated."""))

    title = models.CharField(_("title"), max_length=255)
    slug = AutoSlugField(_("slug"), populate_from="title", unique_with="show__site")

    subtitle = models.CharField(
        _("subtitle"), max_length=255, blank=True,
        help_text=_("Looks best if only a few words like a tagline."))

    description = models.TextField(
        _("description"), max_length=4000, blank=True, help_text=_("""
            This is your chance to tell potential subscribers all about your podcast.
            Describe your subject matter, media format, episode schedule, and other
            relevant info so that they know what they'll be getting when they
            subscribe. In addition, make a list of the most relevant search terms
            that you want your podcast to match, then build them into your
            description. Note that iTunes removes podcasts that include lists of
            irrelevant words in the itunes:summary, description, or
            itunes:keywords tags. This field can be up to 4000 characters."""))
    tracklist = models.TextField(
        _("tracklist"), blank=True,
        help_text=_("""One track per line, machine will automatically add the numbers."""))

    tweet_text = models.CharField(_("tweet text"), max_length=140, editable=False)

    original_image = ImageField(
        _("image"), upload_to=get_episode_upload_folder, blank=True, help_text=_("""
            A podcast must have 1400 x 1400 pixel cover art in JPG or PNG
            format using RGB color space. See our technical spec for
            details. To be eligible for featuring on iTunes Stores,
            choose an attractive, original, and square JPEG (.jpg) or
            PNG (.png) image at a size of 1400x1400 pixels. The image
            will be scaled down to 50x50 pixels at smallest in iTunes.
            For reference see the <a
            href="http://www.apple.com/itunes/podcasts/specs.html#metadata">iTunes
            Podcast specs</a>.<br /><br /> For episode artwork to
            display in iTunes, image must be <a
            href="http://answers.yahoo.com/question/index?qid=20080501164348AAjvBvQ">
            saved to file's <strong>metadata</strong></a> before
            enclosure uploading!"""))

    if ImageSpecField:
        admin_thumb_sm = ImageSpecField(source="original_image",
                                        processors=[ResizeToFill(50, 50)],
                                        options={"quality": 100})
        admin_thumb_lg = ImageSpecField(source="original_image",
                                        processors=[ResizeToFill(450, 450)],
                                        options={"quality": 100})
        img_episode_sm = ImageSpecField(source="original_image",
                                        processors=[ResizeToFill(120, 120)],
                                        options={"quality": 100})
        img_episode_lg = ImageSpecField(source="original_image",
                                        processors=[ResizeToFill(550, 550)],
                                        options={"quality": 100})
        img_itunes_sm = ImageSpecField(source="original_image",
                                       processors=[ResizeToFill(144, 144)],
                                       options={"quality": 100})
        img_itunes_lg = ImageSpecField(source="original_image",
                                       processors=[ResizeToFill(1400, 1400)],
                                       options={"quality": 100})

    # iTunes specific fields
    hours = models.SmallIntegerField(_("hours"), max_length=2, default=0)
    minutes = models.SmallIntegerField(_("minutes"), max_length=2, default=0, choices=SIXTY_CHOICES)
    seconds = models.SmallIntegerField(_("seconds"), max_length=2, default=0, choices=SIXTY_CHOICES)
    keywords = models.CharField(
        _("keywords"), max_length=255, blank=True,
        help_text=_("A comma-delimited list of words for searches, up to 12; "
                    "perhaps include misspellings."))
    explicit = models.PositiveSmallIntegerField(
        _("explicit"), choices=Show.EXPLICIT_CHOICES,
        help_text=_("``Clean`` will put the clean iTunes graphic by it."), default=1)
    block = models.BooleanField(
        _("block"), default=False,
        help_text=_("Check to block this episode from iTunes because <br />its "
                    "content might cause the entire show to be <br />removed from iTunes."""))

    objects = PassThroughManager.for_queryset_class(EpisodeManager)()
    tags = TaggableManager(blank=True)

    class Meta:
        verbose_name = _("Episode")
        verbose_name_plural = _("Episodes")
        ordering = ("-published", "slug")

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse("podcasting_episode_detail",
                       kwargs={"show_slug": self.show.slug, "slug": self.slug})

    def as_tweet(self):
        if not self.tweet_text:
            current_site = Site.objects.get_current()
            api_url = "http://api.tr.im/api/trim_url.json"
            u = urlopen("{0}?url=http://{1}{2}".format(
                api_url,
                current_site.domain,
                self.get_absolute_url(),
            ))
            result = json.loads(u.read())
            self.tweet_text = "{0} {1} - {2}".format(
                self.show.episode_twitter_tweet_prefix,
                self.title,
                result["url"],
            )
        return self.tweet_text

    def tweet(self):
        if can_tweet():
            account = twitter.Api(
                username=settings.TWITTER_USERNAME,
                password=settings.TWITTER_PASSWORD)
            account.PostUpdate(self.as_tweet())
        else:
            raise ImproperlyConfigured(
                "Unable to send tweet due to either "
                "missing python-twitter or required settings.")

    def seconds_total(self):
        try:
            return self.minutes * 60 + self.seconds
        except:
            return 0

    def get_share_url(self):
        return "http://{0}{1}".format(Site.objects.get_current(), self.get_absolute_url())

    def get_share_title(self):
        return self.title

    def get_share_description(self):
        return "{0}...".format(self.description[:512])


@python_2_unicode_compatible
class Enclosure(models.Model):
    """
    An enclosure is one, of possibly many, files/filetypes of an episode.
    """
    MIME_CHOICES = (
        ("aiff", "audio/aiff"),
        ("flac", "audio/flac"),
        ("mp3", "audio/mpeg"),
        ("mp4", "audio/mp4"),
        ("ogg", "audio/ogg"),
        ("flac", "audio/flac"),
        ("wav", "audio/wav"),
    )

    episode = models.ForeignKey(Episode, verbose_name=_("episode"))

    url = models.URLField(
        _("url"),
        help_text=_("""URL of the media file. <br /> It is <strong>very</strong>
            important to remember that for episode artwork to display in iTunes, image must be
            <a href="http://answers.yahoo.com/question/index?qid=20080501164348AAjvBvQ">
            saved to file's <strong>metadata</strong></a> before enclosure uploading!<br /><br />
            For best results, choose an attractive, original, and square JPEG (.jpg) or PNG (.png)
            image at a size of 1400x1400 pixels. The image will be
            scaled down to 50x50 pixels at smallest in iTunes."""))

    size = models.PositiveIntegerField(
        _("size"), help_text=_("The length attribute is the file size in bytes. "
                               "Find this information in the files properties "
                               "(on a Mac, ``Get Info`` and refer to the size row)"))
    mime = models.CharField(
        _("mime format"), max_length=4, choices=MIME_CHOICES,
        help_text=_("Supports mime types of: {0}".format(
            ", ".join([mime[0] for mime in MIME_CHOICES]))))
    bitrate = models.CharField(
        _("bit rate"), max_length=5, default="192",
        help_text=_("Measured in kilobits per second (kbps), often 128 or 192."))
    sample = models.CharField(
        _("sample rate"), max_length=5, default="44.1",
        help_text=_("Measured in kilohertz (kHz), often 44.1."))
    channel = models.CharField(
        _("channel"), max_length=1, default=2,
        help_text=_("Number of channels; 2 for stereo, 1 for mono."))
    duration = models.IntegerField(
        _("duration"), max_length=1,
        help_text=_("Duration of the audio file, in seconds (always as integer)."))

    class Meta:
        ordering = ("episode", "mime")
        unique_together = ("episode", "mime")
        verbose_name = _("Enclosure")
        verbose_name_plural = _("Enclosures")

    def __str__(self):
        return "{} - {}".format(self.episode, self.mime)


@python_2_unicode_compatible
class EmbedMedia(models.Model):
    """
    Associate a media URL to an Episode.

    This is *not* a replacement for a video podcast, but simply a way
    to embed content via url in an episode description.

    Ideally this will be used with django-embed-video which supports
    easy embeding for YouTube and Vimeo videos and music from SoundCloud.
    """
    episode = models.ForeignKey(Episode, verbose_name=_("episode"))

    if EmbedVideoField:
        url = EmbedVideoField(_("url"), help_text=_("URL of the media file"))
    else:
        url = models.URLField(_("url"), help_text=_("URL of the media file"))

    class Meta:
        ordering = ("episode", "url")
        unique_together = ("episode", "url")
        verbose_name = _("Embed Media URL")
        verbose_name_plural = _("Embed Media URLs")

    def __str__(self):
        return "{} - {}".format(self.episode, self.url)



from django.conf.urls import patterns, url
from podcasting.views import ShowListView, ShowDetailView, EpisodeListView, EpisodeDetailView


urlpatterns = patterns(
    "",

    url(r"^$", ShowListView.as_view(),
        name="podcasting_show_list"),
    url(r"^(?P<slug>[-\w]+)/$", ShowDetailView.as_view(),
        name="podcasting_show_detail"),
    url(r"^(?P<show_slug>[-\w]+)/archive/$", EpisodeListView.as_view(),
        name="podcasting_episode_list"),
    url(r"^(?P<show_slug>[-\w]+)/(?P<slug>[-\w]+)/$", EpisodeDetailView.as_view(),
        name="podcasting_episode_detail"),
)



from django.utils.translation import ugettext_lazy as _

from django.contrib import admin

try:
    from imagekit.admin import AdminThumbnail
except ImportError:
    AdminThumbnail = None

from podcasting.forms import AdminShowForm, AdminEpisodeForm
from podcasting.models import Show, Episode, Enclosure, EmbedMedia
from podcasting.utils.twitter import can_tweet


class ShowAdmin(admin.ModelAdmin):
    form = AdminShowForm

    list_display = ["title", "slug", "show_site", "published_flag"]
    list_filter = ["title", "published", "site"]
    if AdminThumbnail:
        list_display.append("admin_thumbnail")
        admin_thumbnail = AdminThumbnail(image_field="admin_thumb_sm")

    if can_tweet():
        fields.append("tweet_text")  # noqa

    def published_flag(self, obj):
        return bool(obj.published)
    published_flag.short_description = _("Published")
    published_flag.boolean = True

    def show_site(self, obj):
        return obj.site.name
    show_site.short_description = "Site"


class EpisodeAdmin(admin.ModelAdmin):
    form = AdminEpisodeForm

    list_display = ["title", "show", "slug", "episode_site", "published_flag"]
    list_filter = ["show", "published"]
    if AdminThumbnail:
        list_display.append("admin_thumbnail")
        admin_thumbnail = AdminThumbnail(image_field="admin_thumb_sm")

    if can_tweet():
        readonly_fields = ("tweet_text",)

    def published_flag(self, obj):
        return bool(obj.published)
    published_flag.short_description = _("Published")
    published_flag.boolean = True

    def episode_site(self, obj):
        return obj.show.site.name
    episode_site.short_description = "Site"

    def save_form(self, request, form, change):
        # this is done for explicitness that we want form.save to commit
        # form.save doesn't take a commit kwarg for this reason
        return form.save()


class EnclosureAdmin(admin.ModelAdmin):
    list_display = ("episode", "mime", "url")
    list_filter = ("episode",)


class EmbedMediaAdmin(admin.ModelAdmin):
    model = EmbedMedia

admin.site.register(Show, ShowAdmin)
admin.site.register(Episode, EpisodeAdmin)
admin.site.register(Enclosure, EnclosureAdmin)
admin.site.register(EmbedMedia, EmbedMediaAdmin)



try:
    from django.utils.timezone import now
except ImportError:
    from datetime.datetime import now  # noqa

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from podcasting.utils.twitter import can_tweet
from podcasting.utils.widgets import CustomAdminThumbnailWidget
from podcasting.models import Enclosure, Episode, Show


class BaseShowForm(forms.ModelForm):

    original_image = forms.ImageField(
        widget=CustomAdminThumbnailWidget,
        help_text=Show._meta.get_field("original_image").help_text)

    publish = forms.BooleanField(
        required=False,
        help_text=_("Checking this will publish this show on the site, no turning back."),
    )

    class Meta:
        model = Show
        fields = [
            "original_image",
            "author_text",
            "owner",
            "editor_email",
            "webmaster_email",
            "title", "subtitle", "description",
            "twitter_tweet_prefix",
            "feedburner", "itunes",
            "keywords", "organization", "license",
            "explicit", "link",
            "on_itunes",
            "publish",
        ]
        if "taggit" in settings.INSTALLED_APPS:
            fields.append("tags")


class ShowAddForm(BaseShowForm):

    def clean_publish(self):
        if self.cleaned_data["publish"]:
            self.instance.published = now()


class ShowChangeForm(BaseShowForm):

    def __init__(self, *args, **kwargs):
        super(ShowChangeForm, self).__init__(*args, **kwargs)
        self.fields["publish"].initial = bool(self.instance.published)

    def clean_publish(self):
        # clean_publish is called twice, skip the first time when instance is unset
        if not self.instance.pk:
            return
        # do nothing if already published
        if self.instance.published:
            return
        if self.cleaned_data["publish"]:
            self.instance.published = now()


class BaseEpisodeForm(forms.ModelForm):

    original_image = forms.ImageField(
        widget=CustomAdminThumbnailWidget,
        help_text=Episode._meta.get_field("original_image").help_text)

    publish = forms.BooleanField(
        required=False,
        help_text=_("Checking this will publish this episode on the site, no turning back."),
    )

    if can_tweet():
        tweet = forms.BooleanField(
            required=False,
            help_text=_("Checking this will send out a tweet announcing the episode."))

    class Meta:
        model = Episode
        fields = [
            "original_image",
            "author_text",
            "title", "subtitle",
            "description",
            "tracklist",
            "hours", "minutes", "seconds",
            "publish",
        ]
        if "taggit" in settings.INSTALLED_APPS:
            fields.append("tags")
        extra_fields_itunes = [
            "keywords",
            "explicit",
            "block",
        ]
        required_fields_itunes = [
            "author_text",
            "subtitle",
            "description",
            "original_image",
            "keywords",
            "explicit",
        ]

    def save(self):
        instance = super(BaseEpisodeForm, self).save()

        if can_tweet() and self.cleaned_data["tweet"]:
            instance.tweet()

        return instance

    def validate_published(self):
        if not self.instance.enclosure_set.count() or not self.instance.embedmedia_set.count():
            raise forms.ValidationError(
                _("An episode must have at least one enclosure or media file before publishing.\n "
                  "Uncheck, save this episode, and add an encoslure before publishing."))
        elif not self.instance.show.published:
            raise forms.ValidationError(_("The show for this episode is not yet published"))
        self.instance.published = now()


class EpisodeChangeForm(BaseEpisodeForm):

    def __init__(self, *args, **kwargs):
        super(EpisodeChangeForm, self).__init__(*args, **kwargs)
        self.fields["publish"].initial = bool(self.instance.published)

    def clean_publish(self):
        # clean_publish is called twice, skip the first time when instance is unset
        if not self.instance.pk:
            return
        # do nothing if already published
        if self.instance.published:
            return
        if self.cleaned_data["publish"]:
            self.validate_published()


class EpisodeITunesChangeForm(EpisodeChangeForm):
    def __init__(self, *args, **kwargs):
        super(EpisodeITunesChangeForm, self).__init__(*args, **kwargs)
        for key in self.Meta.required_fields_itunes:
            self.fields[key].required = True

    class Meta(EpisodeChangeForm.Meta):
        fields = EpisodeChangeForm.Meta.fields + EpisodeChangeForm.Meta.extra_fields_itunes


class EpisodeAddForm(BaseEpisodeForm):

    def clean_publish(self):
        if self.cleaned_data["publish"]:
            self.validate_published()


class EpisodeITunesAddForm(EpisodeAddForm):
    def __init__(self, *args, **kwargs):
        super(EpisodeITunesAddForm, self).__init__(*args, **kwargs)
        for key in self.Meta.required_fields_itunes:
            self.fields[key].required = True

    class Meta(EpisodeAddForm.Meta):
        fields = EpisodeAddForm.Meta.fields + EpisodeAddForm.Meta.extra_fields_itunes


class EnclosureForm(forms.ModelForm):

    class Meta:
        model = Enclosure
        fields = [
            "url",
            "mime",
            "size",
            "bitrate",
            "sample",
            "channel",
        ]

    def validate_unique(self):
        exclude = self._get_validation_exclusions()
        exclude.remove("episode")  # allow checking against the missing attribute

        try:
            self.instance.validate_unique(exclude=exclude)
        except forms.ValidationError as err:
            self._update_errors(err.message_dict)


class AdminShowForm(forms.ModelForm):

    publish = forms.BooleanField(
        label=_("publish"),
        required=False,
        help_text=_("Checking this will publish this show on the site, no turning back."),
    )

    class Meta:
        model = Show
        fields = [
            "site",
            "original_image",
            "author_text",
            "owner",
            "editor_email",
            "webmaster_email",
            "title", "subtitle", "description",
            "twitter_tweet_prefix",
            "feedburner", "itunes",
            "keywords", "organization", "license",
            "explicit", "link",
            "on_itunes",
            "publish",
        ]
        if "taggit" in settings.INSTALLED_APPS:
            fields.append("tags")

    def __init__(self, *args, **kwargs):
        super(AdminShowForm, self).__init__(*args, **kwargs)
        self.fields["publish"].initial = bool(self.instance.published)

    def clean_publish(self):
        # clean_publish is called twice, skip the first time when instance is unset
        if not self.instance.pk:
            return
        # do nothing if already published
        if self.instance.published:
            return
        if self.cleaned_data["publish"]:
            self.instance.published = now()


class AdminEpisodeForm(forms.ModelForm):

    publish = forms.BooleanField(
        label=_("publish"),
        required=False,
        help_text=_("Checking this will publish this episode on the site, no turning back."))

    if can_tweet():
        tweet = forms.BooleanField(
            required=False,
            help_text=_("Checking this will send out a tweet announcing the episode."))

    class Meta:
        model = Episode
        fields = [
            "show",
            "original_image",
            "author_text",
            "title", "subtitle",
            "description",
            "tracklist",
            "hours", "minutes", "seconds",
            "publish",
            "keywords",
            "explicit",
            "block",
        ]
        if "taggit" in settings.INSTALLED_APPS:
            fields.append("tags")

    def __init__(self, *args, **kwargs):
        super(AdminEpisodeForm, self).__init__(*args, **kwargs)
        self.fields["publish"].initial = bool(self.instance.published)

    def validate_published(self):
        if not self.instance.enclosure_set.count() and not self.instance.embedmedia_set.count():
            raise forms.ValidationError(
                _("An episode must have at least one enclosure or media file before publishing.\n "
                  "Uncheck, save this episode, and add an encoslure before publishing."))
        elif not self.instance.show.published:
            raise forms.ValidationError(_("The show for this episode is not yet published"))
        self.instance.published = now()

    def clean_publish(self):
        # clean_publish is called twice, skip the first time when instance is unset
        if not self.instance.pk:
            return
        # do nothing if already published
        if self.instance.published:
            return
        if self.cleaned_data["publish"]:
            self.validate_published()

    def save(self):
        episode = super(AdminEpisodeForm, self).save(commit=False)

        episode.save()

        if can_tweet() and self.cleaned_data["tweet"]:
            episode.tweet()

        return episode



from __future__ import unicode_literals

# See http://djangosnippets.org/snippets/1580/
from django.forms import ClearableFileInput
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string

try:
    import imagekit
    easy_thumbnails = False
    sorl = False
except:
    pass

try:
    import easy_thumbnails
    imagekit = False  # noqa
    sorl = False
except:
    pass

try:
    import sorl
    imagekit = False
    easy_thumbnails = False  # noqa
except:
    pass


class CustomAdminThumbnailWidget(ClearableFileInput):
    """
    A ImageField Widget that displays a 30px thumbnail on the change form.
    """
    def __init__(self, attrs={}):
        super(CustomAdminThumbnailWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        output = []

        if value and imagekit:
            thumb_sm_url = value.instance.admin_thumb_sm.url
            thumb_lg_url = value.instance.admin_thumb_lg.url
            output.append(render_to_string("podcasting/admin_thumbnail_imagekit.html", {
                "original_image_url": value.url,
                "thumb_sm_url": thumb_sm_url,
                "thumb_lg_url": thumb_lg_url,
            }))
        elif value and easy_thumbnails:
            output.append(render_to_string("podcasting/admin_thumbnail_easy-thumbnails.html", {
                "original_image": value,
            }))
        elif value and sorl:
            output.append(render_to_string("podcasting/admin_thumbnail_sorl.html", {
                "original_image": value,
            }))
        else:
            output.append(render_to_string("podcasting/admin_thumbnail_css.html", {
                "original_image": value,
            }))
        output.append(super(CustomAdminThumbnailWidget, self).render(name, value, attrs))
        return mark_safe("".join(output))



from django.views.generic import DetailView, ListView

from podcasting.conf import settings
from podcasting.models import Episode, Show


class ShowListView(ListView):

    paginate_by = settings.PODCASTING_PAGINATE_BY

    def get_queryset(self):
        return Show.objects.onsite()


class ShowDetailView(DetailView):
    def get_queryset(self):
        return Show.objects.onsite()


class EpisodeListView(ListView):

    paginate_by = settings.PODCASTING_PAGINATE_BY

    def get_queryset(self):
        return Episode.objects.published().filter(show__slug=self.kwargs["show_slug"])


class EpisodeDetailView(DetailView):
    def get_queryset(self):
        return Episode.objects.published().filter(show__slug=self.kwargs["show_slug"])



import factory

from django.test import TestCase

from django.contrib.sites.models import Site
from django.contrib.auth.models import User

try:
    from licenses .models import License
except ImportError:
    License = False

from podcasting.models import Show, Episode, Enclosure


class UserFactory(factory.django.DjangoModelFactory):
    FACTORY_FOR = User


class SiteFactory(factory.django.DjangoModelFactory):
    FACTORY_FOR = Site


if License:
    class LicenseFactory(factory.django.DjangoModelFactory):
        FACTORY_FOR = License


class ShowFactory(factory.django.DjangoModelFactory):
    FACTORY_FOR = Show
    owner = factory.SubFactory(UserFactory)
    site = factory.SubFactory(SiteFactory)
    if License:
        license = factory.SubFactory(LicenseFactory)


class EpisodeFactory(factory.django.DjangoModelFactory):
    FACTORY_FOR = Episode
    show = factory.SubFactory(ShowFactory)


class EnclosureFactory(factory.django.DjangoModelFactory):
    FACTORY_FOR = Enclosure
    episode = factory.SubFactory(EpisodeFactory)
    size = 303
    duration = 909


class PodcastTests(TestCase):
    def setUp(self):
        self.show = ShowFactory.create(title="snowprayers")
        self.show.save()
        self.episodes = []
        for i in range(0, 10):
            episode = EpisodeFactory.create(show=self.show, title="Episode 1")
            self.episodes.append(episode)
        self.episode = EpisodeFactory.create(show=self.show, title="Episode")
        long_title = "".join(["x" for i in range(51)])
        self.long_episode1 = EpisodeFactory.create(show=self.show, title=long_title)

        self.long_episode2 = EpisodeFactory.create(show=self.show, title=long_title)

        self.enclosure = EnclosureFactory.create(episode=self.episodes[0])

    def test_podcast(self):
        self.assertEquals(self.show, self.enclosure.episode.show)

    def test_autoslug(self):
        """Test normal slug generation. Slug has to be lower case."""
        self.assertEqual(self.episode.get_absolute_url(),
                         "/podcasts/snowprayers/episode/",
                         "Slug not generated as expected.")
        self.episode.save()
        self.assertEqual(self.episode.get_absolute_url(),
                         "/podcasts/snowprayers/episode/",
                         "Slug changed after additional episode.save()!")

    def test_slug_append_int_if_already_exists(self):
        """Test that the slug is created properly
        If the slug already exists a '-incremented int' is appended.
        """
        self.assertEquals(self.show.get_absolute_url(),
                          "/podcasts/snowprayers/")
        self.assertEquals(self.episodes[0].get_absolute_url(),
                          "/podcasts/snowprayers/episode-1/")
        for i in range(1, 10):
            self.assertEqual(self.episodes[i].get_absolute_url(),
                             "/podcasts/snowprayers/episode-1-{0}/".format(i + 1))

    def test_shortend_slug(self):
        """Test if slug get's shortened to max_length"""
        self.assertTrue(len(self.long_episode1.slug) == 50,
                        "Length of slug not <= 50 char!")
        self.assertTrue(len(self.long_episode2.slug) == 50,
                        "Length of slug not <= 50 char!")
        self.assertTrue(self.long_episode2.slug.endswith("-2"),
                        "Slug doesn't end with '-2' since same title does exist"
                        " in other episode already!")



from django.conf.urls import patterns, url

from podcasting.feeds import (
    RssShowFeed, AtomShowFeed, AtomRedirectView, RssRedirectView)
from podcasting.models import Enclosure


MIMES = "|".join([enclosure[0] for enclosure in Enclosure.MIME_CHOICES])


urlpatterns = patterns(
    "",

    # Episode list feed by show (RSS 2.0 and iTunes)
    url(r"^(?P<show_slug>[-\w]+)/(?P<mime_type>{mimes})/rss/$".format(mimes=MIMES),
        RssShowFeed(), name="podcasts_show_feed_rss"),

    # Episode list feed by show (Atom)
    url(r"^(?P<show_slug>[-\w]+)/(?P<mime_type>{mimes})/atom/$".format(mimes=MIMES),
        AtomShowFeed(), name="podcasts_show_feed_atom"),

    # Episode list feed by show (Media RSS)
    # TODO upon request

    # Previously we had /itunes/ in the feed url.
    # This is now deprecated and redirects to a more general feed url.
    url(r"^(?P<show_slug>[-\w]+)/itunes/(?P<mime_type>[-\w]+)/rss/$",
        RssRedirectView.as_view()),
    url(r"^(?P<show_slug>[-\w]+)/itunes/(?P<mime_type>[-\w]+)/atom/$",
        AtomRedirectView.as_view()),
)



from django.conf import settings

try:
    import twitter
except ImportError:
    twitter = None  # noqa


def can_tweet():
    creds_available = (hasattr(settings, "TWITTER_USERNAME") and
                       hasattr(settings, "TWITTER_PASSWORD"))
    return twitter and creds_available



from django.db.models import Q
from django.db.models.query import QuerySet

from django.contrib.sites.models import Site


class EpisodeManager(QuerySet):
    """Returns public episodes that are currently activated."""

    def itunespublished(self):
        return self.get_queryset().exclude(Q(published=None) | Q(block=True))

    def published(self):
        return self.exclude(published=None)

    def onsite(self):
        return self.filter(show__site=Site.objects.get_current())

    def current(self):
        try:
            return self.published().order_by("-published")[0]
        except IndexError:
            return None


class ShowManager(QuerySet):
    """Returns shows that are on the current site."""

    def published(self):
        return self.exclude(published=None)

    def onsite(self):
        return self.filter(site=Site.objects.get_current())



# If you change this version, change it also in docs/conf.py
# Packging guide: http://guide.python-distribute.org/quickstart.html
__version_info__ = {
    "major": 0,
    "minor": 9,
    "micro": 10,
    "releaselevel": "final",
    "serial": 0,
}


def get_version():
    vers = ["{major}.{minor}".format(**__version_info__)]

    if __version_info__["micro"]:
        vers.append(".{micro}".format(**__version_info__))
    if __version_info__["releaselevel"] != "final":
        vers.append("{releaselevel}{serial}".format(**__version_info__))
    return "".join(vers)


__version__ = get_version()



"""
Django Extensions model fields
"""
from __future__ import unicode_literals

import uuid

from django.db.models import CharField


class UUIDVersionError(Exception):
    pass


class UUIDField(CharField):
    """UUIDField

    By default uses UUID version 1 (generate from host ID, sequence
    number and current time)

    The field support all uuid versions which are natively supported
    by the uuid python module. For more information see:
    http://docs.python.org/lib/module-uuid.html

    """

    def __init__(self, verbose_name=None, name=None, auto=True,
                 version=1, node=None, clock_seq=None, namespace=None, **kwargs):
        kwargs['max_length'] = 36
        if auto:
            kwargs['blank'] = True
            kwargs.setdefault('editable', False)
        self.auto = auto
        self.version = version
        if version == 1:
            self.node, self.clock_seq = node, clock_seq
        elif version == 3 or version == 5:
            self.namespace, self.name = namespace, name
        CharField.__init__(self, verbose_name, name, **kwargs)

    def get_internal_type(self):
        return CharField.__name__

    def contribute_to_class(self, cls, name):
        if self.primary_key:
            assert not cls._meta.has_auto_field, "A model can't have more than one\
                AutoField: %s %s %s; have %s" % (self, cls, name, cls._meta.auto_field)
            super(UUIDField, self).contribute_to_class(cls, name)
            cls._meta.has_auto_field = True
            cls._meta.auto_field = self
        else:
            super(UUIDField, self).contribute_to_class(cls, name)

    def create_uuid(self):
        if not self.version or self.version == 4:
            return uuid.uuid4()
        elif self.version == 1:
            return uuid.uuid1(self.node, self.clock_seq)
        elif self.version == 2:
            raise UUIDVersionError("UUID version 2 is not supported.")
        elif self.version == 3:
            return uuid.uuid3(self.namespace, self.name)
        elif self.version == 5:
            return uuid.uuid5(self.namespace, self.name)
        else:
            raise UUIDVersionError("UUID version %s is not valid." % self.version)

    def pre_save(self, model_instance, add):
        if self.auto and add:
            value = self.create_uuid()
            setattr(model_instance, self.attname, value)
            return value
        else:
            value = super(UUIDField, self).pre_save(model_instance, add)
            if self.auto and not value:
                value = self.create_uuid()
                setattr(model_instance, self.attname, value)
        return value

    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.CharField"
        args, kwargs = introspector(self)
        # That's our definition!
        return (field_class, args, kwargs)



import os
import sys
from fnmatch import fnmatchcase
from distutils.util import convert_path
from setuptools import setup, find_packages


# Provided as an attribute, so you can append to these instead
# of replicating them:
standard_exclude = ("*.py", "*.pyc", "*$py.class", "*~", ".*", "*.bak")
standard_exclude_directories = (".*", "CVS", "_darcs", "./build",
                                "./dist", "EGG-INFO", "*.egg-info")

# (c) 2005 Ian Bicking and contributors; written for Paste (http://pythonpaste.org)
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
# Note: you may want to copy this into your setup.py file verbatim, as
# you can't import this from another package, when you don't know if
# that package is installed yet.


def find_package_data(where=".", package="",
                      exclude=standard_exclude,
                      exclude_directories=standard_exclude_directories,
                      only_in_packages=True,
                      show_ignored=False):
    """\
Return a dictionary suitable for use in ``package_data``
in a distutils ``setup.py`` file.

The dictionary looks like::

{"package": [files]}

Where ``files`` is a list of all the files in that package that
don"t match anything in ``exclude``.

If ``only_in_packages`` is true, then top-level directories that
are not packages won't be included (but directories under packages
will).

Directories matching any pattern in ``exclude_directories`` will
be ignored; by default directories with leading ``.``, ``CVS``,
and ``_darcs`` will be ignored.

If ``show_ignored`` is true, then all the files that aren't
included in package data are shown on stderr (for debugging
purposes).

Note patterns use wildcards, or can be exact paths (including
leading ``./``), and all searching is case-insensitive.
"""

    out = {}
    stack = [(convert_path(where), "", package, only_in_packages)]
    while stack:
        where, prefix, package, only_in_packages = stack.pop(0)
        for name in os.listdir(where):
            fn = os.path.join(where, name)
            if os.path.isdir(fn):
                bad_name = False
                for pattern in exclude_directories:
                    if (fnmatchcase(name, pattern) or fn.lower() == pattern.lower()):
                        bad_name = True
                        if show_ignored:
                            print >> sys.stderr, (
                                "Directory %s ignored by pattern %s"
                                % (fn, pattern))
                        break
                if bad_name:
                    continue
                if (os.path.isfile(os.path.join(fn, "__init__.py")) and not prefix):
                    if not package:
                        new_package = name
                    else:
                        new_package = package + "." + name
                    stack.append((fn, "", new_package, False))
                else:
                    stack.append((fn, prefix + name + "/", package, only_in_packages))
            elif package or not only_in_packages:
                # is a file
                bad_name = False
                for pattern in exclude:
                    if (fnmatchcase(name, pattern) or fn.lower() == pattern.lower()):
                        bad_name = True
                        if show_ignored:
                            print >> sys.stderr, (
                                "File %s ignored by pattern %s"
                                % (fn, pattern))
                        break
                if bad_name:
                    continue
                out.setdefault(package, []).append(prefix + name)
    return out


VERSION = __import__("podcasting").__version__


setup(
    name="django-podcasting",
    version=VERSION,
    url="http://django-podcasting.readthedocs.org/",
    license="BSD",
    description="Audio podcasting functionality for django sites.",
    long_description=open("README.rst").read(),
    author="Thomas Schreiber",
    author_email="tom@nillab.com",
    packages=find_packages(),
    package_data=find_package_data("podcasting", only_in_packages=False),
    install_requires=[
        "django-model-utils>=2.0",
        "django-appconf>=0.6",
    ],
    tests_require=[
        "Django>=1.4",
        "django-autoslug>=1.7.2",
        "Pillow>=2.3.0",
        "factory_boy>=2.3.0",
    ],
    test_suite="runtests.runtests",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    zip_safe=False,
)



from __future__ import unicode_literals

import datetime

from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.feedgenerator import rfc2822_date, Rss201rev2Feed, Atom1Feed
from django.shortcuts import get_object_or_404

from django.contrib.sites.models import get_current_site
from django.contrib.syndication.views import Feed
from django.views.generic.base import RedirectView

try:
    import imagekit
    easy_thumbnails = False
    sorl = False
except ImportError:
    pass

try:
    import easy_thumbnails
    imagekit = False  # noqa
    sorl = False
except ImportError:
    pass

try:
    import sorl
    imagekit = False
    easy_thumbnails = False  # noqa
except ImportError:
    pass

try:
    import licenses
except ImportError:
    licenses = False

from podcasting.models import Enclosure, Show
from podcasting.conf import settings as settings2


class ITunesElements(object):

    def add_root_elements(self, handler):
        """ Add additional elements to the show object"""
        super(ITunesElements, self).add_root_elements(handler)

        show = self.feed["show"]

        if show.original_image:
            if imagekit:
                itunes_sm_url = show.img_itunes_sm.url
                itunes_lg_url = show.img_itunes_lg.url
            elif easy_thumbnails:
                aliases = settings.THUMBNAIL_ALIASES["podcasting.Show.original_image"]
                thumbnailer = easy_thumbnails.files.get_thumbnailer(show.original_image)
                try:
                    itunes_sm_url = thumbnailer.get_thumbnail(aliases["itunes_sm"]).url
                    itunes_lg_url = thumbnailer.get_thumbnail(aliases["itunes_lg"]).url
                except easy_thumbnails.exceptions.InvalidImageFormatError:
                    easy_thumbnails.signal_handlers.generate_aliases_global(show.original_image)
                    itunes_sm_url = thumbnailer.get_thumbnail(aliases["itunes_sm"]).url
                    itunes_lg_url = thumbnailer.get_thumbnail(aliases["itunes_lg"]).url
                except AttributeError:
                    itunes_sm_url = None
                    itunes_lg_url = None
            elif sorl:
                itunes_sm_url = sorl.thumbnail.get_thumbnail(show.original_image, "144x144").url
                itunes_lg_url = sorl.thumbnail.get_thumbnail(show.original_image, "1400x1400").url
            else:
                itunes_sm_url = show.original_image.url
                itunes_lg_url = show.original_image.url
            if itunes_sm_url and itunes_lg_url:
                handler.addQuickElement("itunes:image", attrs={"href": itunes_lg_url})
                handler.startElement("image", {})
                handler.addQuickElement("url", itunes_sm_url)
                handler.addQuickElement("title", self.feed["title"])
                handler.addQuickElement("link", self.feed["link"])
                handler.endElement("image")

        handler.addQuickElement("guid", str(show.uuid), attrs={"isPermaLink": "false"})
        handler.addQuickElement("itunes:subtitle", self.feed["subtitle"])
        handler.addQuickElement("itunes:author", show.author_text)
        handler.startElement("itunes:owner", {})
        handler.addQuickElement("itunes:name", show.owner.get_full_name())
        handler.addQuickElement("itunes:email", show.owner.email)
        handler.endElement("itunes:owner")
        handler.addQuickElement("itunes:category", attrs={"text": self.feed["categories"][0]})
        handler.addQuickElement("itunes:summary", show.description)
        handler.addQuickElement("itunes:explicit", show.get_explicit_display())
        if show.redirect:
            handler.addQuickElement("itunes:new-feed-url", show.redirect)
        handler.addQuickElement("keywords", show.keywords)
        if show.editor_email:
            handler.addQuickElement("managingEditor", show.editor_email)
        if show.webmaster_email:
            handler.addQuickElement("webMaster", show.webmaster_email)
        try:
            handler.addQuickElement("lastBuildDate",
                                    rfc2822_date(show.episode_set.published()[1].published))
        except IndexError:
            pass
        handler.addQuickElement("generator", "Django Web Framework")
        handler.addQuickElement("docs", "http://blogs.law.harvard.edu/tech/rss")

    def add_item_elements(self, handler, item):
        """ Add additional elements to the episode object"""
        super(ITunesElements, self).add_item_elements(handler, item)

        episode = item["episode"]
        if episode.original_image:
            if imagekit:
                itunes_sm_url = episode.img_itunes_sm.url
                itunes_lg_url = episode.img_itunes_lg.url
            elif easy_thumbnails:
                aliases = settings.THUMBNAIL_ALIASES["podcasting.Episode.original_image"]
                thumbnailer = easy_thumbnails.files.get_thumbnailer(episode.original_image)
                try:
                    itunes_sm_url = thumbnailer.get_thumbnail(aliases["itunes_sm"]).url
                    itunes_lg_url = thumbnailer.get_thumbnail(aliases["itunes_lg"]).url
                except easy_thumbnails.exceptions.InvalidImageFormatError:
                    easy_thumbnails.signal_handlers.generate_aliases_global(episode.original_image)
                    itunes_sm_url = thumbnailer.get_thumbnail(aliases["itunes_sm"]).url
                    itunes_lg_url = thumbnailer.get_thumbnail(aliases["itunes_lg"]).url
                except AttributeError:
                    itunes_sm_url = None
                    itunes_lg_url = None
            elif sorl:
                itunes_sm_url = sorl.thumbnail.get_thumbnail(episode.original_image, "144x144").url
                itunes_lg_url = sorl.thumbnail.get_thumbnail(episode.original_image, "1400x1400").url  # noqa
            else:
                itunes_sm_url = episode.original_image.url
                itunes_lg_url = episode.original_image.url
            if itunes_sm_url and itunes_lg_url:
                handler.addQuickElement("itunes:image", attrs={"href": itunes_lg_url})
                handler.startElement("image", {})
                handler.addQuickElement("url", itunes_sm_url)
                handler.addQuickElement("title", episode.title)
                handler.addQuickElement("link", episode.get_absolute_url())
                handler.endElement("image")

        handler.addQuickElement("guid", str(episode.uuid), attrs={"isPermaLink": "false"})
        if licenses:
            handler.addQuickElement("copyright", "{0} {1} {2}".format(episode.show.license.name,
                                                                      episode.show.license.url,
                                                                      datetime.date.today().year))
        else:
            handler.addQuickElement("copyright", "{0} {1}".format(episode.show.license.name,
                                                                  datetime.date.today().year))
        handler.addQuickElement("itunes:author", episode.author_text)
        handler.addQuickElement("itunes:subtitle", episode.subtitle)
        handler.addQuickElement("itunes:summary", episode.description)
        handler.addQuickElement("itunes:duration", "%02d:%02d:%02d" % (episode.hours,
                                                                       episode.minutes,
                                                                       episode.seconds))
        handler.addQuickElement("itunes:keywords", episode.keywords)
        handler.addQuickElement("itunes:explicit", episode.get_explicit_display())
        if episode.block:
            handler.addQuickElement("itunes:block", "yes")

    def namespace_attributes(self):
        return {"xmlns:itunes": "http://www.itunes.com/dtds/podcast-1.0.dtd"}


class AtomITunesFeedGenerator(ITunesElements, Atom1Feed):
    def root_attributes(self):
        atom_attrs = super(AtomITunesFeedGenerator, self).root_attributes()
        atom_attrs.update(self.namespace_attributes())
        return atom_attrs


class RssITunesFeedGenerator(ITunesElements, Rss201rev2Feed):
    def rss_attributes(self):
        rss_attrs = super(RssITunesFeedGenerator, self).rss_attributes()
        rss_attrs.update(self.namespace_attributes())
        return rss_attrs


class ShowFeed(Feed):
    """
    A feed of podcasts for iTunes and other compatible podcatchers.
    """
    def title(self, show):
        return show.title

    def link(self, show):
        return show.link

    def categories(self, show):
        return ("Music",)

    def feed_copyright(self, show):
        if licenses:
            return "{0} {1} {2}".format(
                show.license.name, show.license.url, datetime.date.today().year)
        else:
            return "{0} {1}".format(show.license, datetime.date.today().year)

    def ttl(self, show):
        return show.ttl

    def items(self, show):
        return show.episode_set.published()[:settings2.PODCASTING_FEED_ENTRIES]

    def get_object(self, request, *args, **kwargs):
        self.mime = [mc[0] for mc in Enclosure.MIME_CHOICES if mc[0] == kwargs["mime_type"]][0]
        site = get_current_site(request)
        self.show = get_object_or_404(Show, slug=kwargs["show_slug"], site=site)
        return self.show

    def item_title(self, episode):
        return episode.title

    def item_description(self, episode):
        "renders summary for atom"
        return episode.description

    def item_link(self, episode):
        return reverse("podcasting_episode_detail",
                       kwargs={"show_slug": episode.show.slug, "slug": episode.slug})

    # def item_author_link(self, episode):
    #     return "todo" #this one doesn't add anything in atom or rss
    #
    # def item_author_email(self, episode):
    #     return "todo" #this one doesn't add anything in atom or rss

    def item_pubdate(self, episode):
        return episode.published

    def item_categories(self, episode):
        return self.categories(episode.show)

    def item_enclosure_url(self, episode):
        try:
            e = episode.enclosure_set.get(mime=self.mime)
            return e.url
        except Enclosure.DoesNotExist:
            pass

    def item_enclosure_length(self, episode):
        try:
            e = episode.enclosure_set.get(mime=self.mime)
            return e.size
        except Enclosure.DoesNotExist:
            pass

    def item_enclosure_mime_type(self, episode):
        try:
            e = episode.enclosure_set.get(mime=self.mime)
            return e.get_mime_display()
        except Enclosure.DoesNotExist:
            pass

    def item_keywords(self, episode):
        return episode.keywords

    def feed_extra_kwargs(self, obj):
        extra = {}
        extra["show"] = self.show
        return extra

    def item_extra_kwargs(self, item):
        extra = {}
        extra["episode"] = item
        return extra


class AtomShowFeed(ShowFeed):
    feed_type = AtomITunesFeedGenerator

    def subtitle(self, show):
        return show.subtitle

    def author_name(self, show):
        return show.owner.get_full_name()

    def author_email(self, show):
        return show.owner.email

    def author_link(self, show):
        return show.link


class RssShowFeed(ShowFeed):
    feed_type = RssITunesFeedGenerator

    def item_guid(self, episode):
        "ITunesElements can't add isPermaLink attr unless None is returned here."
        return None

    def description(self, show):
        return show.description


class AtomRedirectView(RedirectView):
    permanent = False

    def get_redirect_url(self, show_slug, mime_type):
        return reverse(
            "podcasts_show_feed_atom",
            kwargs={"show_slug": show_slug, "mime_type": mime_type})


class RssRedirectView(RedirectView):
    permanent = False

    def get_redirect_url(self, show_slug, mime_type):
        return reverse(
            "podcasts_show_feed_rss",
            kwargs={"show_slug": show_slug, "mime_type": mime_type})
